''8-Bit Game Graphics tile driver
'' JT Cook
''

CON
  X_Length = 32 ''number of tiles that run horizontally across screen
  Y_Length = 24 ''number of tiles that run vertically across screen

' constants
 SCANLINE_BUFFER = $7F00           
 
 request_scanline       = SCANLINE_BUFFER-4  'address of scanline buffer for TV driver
 tilemap_adr            = SCANLINE_BUFFER-8  'address of tile map
 tile_adr               = SCANLINE_BUFFER-12 'address of tiles
 border_color           = SCANLINE_BUFFER-16 'address of border color 
 sprite_adr             = SCANLINE_BUFFER-20  'address of where sprites are stored
 sprite_x_adr           = SCANLINE_BUFFER-24 'X location of 8 sprites
 sprite_y_adr           = SCANLINE_BUFFER-32 'Y location of 8 sprites
 sprite_number          = SCANLINE_BUFFER-40 'address of images of 8 sprites
 sprite_x_flip          = SCANLINE_BUFFER-48 'flag to flip sprite on x axis
 sprite_y_flip          = SCANLINE_BUFFER-49 'flag to flip sprite on y axis
 onebpp_tile_adr        = SCANLINE_BUFFER-50 'tile map for 1bpp tiles
 sprite_color_01        = SCANLINE_BUFFER-58
 sprite_color_10        = SCANLINE_BUFFER-66
 tile_Pallete_adr       = SCANLINE_BUFFER-70 'address of tiles


 x_tiles = 16 '*16=240
 y_tiles = 15 '13 '*16=160
 'colormode = 0


OBJ
  'tv    : "JB_tv_02.spin"             ' tv driver 256 pixel scanline
  tv    : "waitvid.256x192.driver.2048.spin"
  'tv    : "VGA8_256x192_v24.spin"
  gfx   : "JTC_Tile_Renderer.spin"    ' graphics engine

VAR
   byte Tile_Map[X_Length*Y_Length] ''tile map
   byte Scr_buff[X_Length*Y_Length]
   byte Color0[X_Length*Y_Length]
   byte Color1[X_Length*Y_Length]
      'byte Tiles[(64)*Num_Of_Tiles] ''number of tiles (8x8 pixels = 64)

   long cog_number ''used for rendering engine
   long cog_total  ''used for rendering engine  
   'long border_color ''used for borders
''used for TV driver
   long tv_status      '0/1/2 = off/visible/invisible           read-only
   long tv_enable      '0/? = off/on                            write-only
   long tv_pins        '%ppmmm = pins                           write-only
   long tv_mode        '%ccinp = chroma,interlace,ntsc/pal,swap write-only
   long tv_screen      'pointer to screen (words)               write-only
   long tv_colors      'pointer to colors (longs)               write-only               
   long tv_hc          'horizontal cells                        write-only
   long tv_vc          'vertical cells                          write-only
   long tv_hx          'horizontal cell expansion               write-only
   long tv_vx          'vertical cell expansion                 write-only
   long tv_ho          'horizontal offset                       write-only
   long tv_vo          'vertical offset                         write-only
   long tv_broadcast   'broadcast frequency (Hz)                write-only
   long tv_auralcog    'aural fm cog                            write-only
''used to stop and start tv driver


'PUB tv_start
  'tv.start(@tvparams)
'  tv.start(@vga_params)
  

PUB tv_stop
   'tv.stop
   
PUB start(video_pins,NorP, cog_num, colormode)      | i
  DIRA[0] := 1
  outa[0] := 0

  'tv.start(@vga_params) 
  tv.init(-1,$7f00,colormode)
  'if ColorMode == 0
  '   long[@VgaDriver][288] := %0000_0011_0000_0011_0000_0000_0000_0000
  'else
  '   long[@VgaDriver][289] := %0_01_1_0_0_000_00000100000_011_0_00000011  
  'cognew(@VgaDriver, $7f00)
  
  long[tilemap_adr] := @Tile_Map  'address of tile map
  long[@tvparams+8]:=video_pins ''map pins for video out
  long[@tvparams+12]:=NorP ''NTSC or PAL  
  ' Boot requested number of rendering cogs:
  ' this must be at least 4, anything less will not be enough cogs  
  'cog_total := 3
  cog_total :=cog_num
  cog_number := 0
  repeat
    gfx.start(@cog_number)
    
    repeat 1000 ' Allow some time for previous cog to boot up before setting 'cog_number' again
    cog_number++
  until cog_number == cog_total  
  long[border_color]:=$02 ''default border color
  'start tv driver
  'tv.start(@tvparams)

PUB Wait_Vsync ''wait until frame is done drawing
    repeat while long[$7F00-4] <> 191 '+ 48 '16
    
PUB Set_Border_Color(bcolor) | i ''set the color for border around screen
    long[border_color]:=bcolor
    
PUB Poke_Tile_XY(x_location, y_location, tile)
'' Place a tile in the tilemap
 ''x_location - select x position of tile
 ''y_location - select y position of tile
 ''tile - which tile will occupy location
   y_location<<=5 '' multiply by 32 since there are 32 tiles per row
   x_location+=y_location ''get final address
   Tile_Map[x_location]:=tile

PUB Peek_Tile_XY(x_location, y_location) : tile
   y_location<<=5 '' multiply by 32 since there are 32 tiles per row
   x_location+=y_location ''get final address
   tile:= Tile_Map[x_location]

pub Poke_Color_XY(x_location, y_location, clr0, clr1)
   y_location<<=5 '' multiply by 32 since there are 32 tiles per row
   x_location+=y_location ''get final address
   Color0[x_location]:=clr0
   Color1[x_location]:=clr1
          
pub Poke_Color(index, clr0, clr1)
   Color0[index]:=clr0
   Color1[index]:=clr1

pub Poke_Tile(index, tile)
   Tile_Map[index] := tile
   
pub Poke_buff(index, tile)
   Scr_buff[index] := tile

pub UpdateScreen
    longmove(@Tile_Map, @Scr_buff, 192)


pub Spr_Setup(Spr_Number, x_location, y_location, Spr_Image, Color_01, Color_10)
    byte[sprite_x_adr-Spr_Number] := x_location
    byte[sprite_y_adr-Spr_Number] := y_location
    byte[sprite_color_01-Spr_Number] := Color_01
    byte[sprite_color_10-Spr_Number] := Color_10

PUB Spr_Flip_X(sprite_nbr)
          BYTE[sprite_x_flip]|=(1<<sprite_nbr)

PUB Spr_UnFlip_X(sprite_nbr)
        BYTE[sprite_x_flip]&=($FF-(1<<sprite_nbr))

PUB Spr_Flip_Y(sprite_nbr)
          BYTE[sprite_y_flip]|=(1<<sprite_nbr)

PUB Spr_UnFlip_Y(sprite_nbr)
        BYTE[sprite_y_flip]&=($FF-(1<<sprite_nbr))

pub Spr_Get_X(Spr_Number)
  return byte[sprite_x_adr-Spr_Number]

pub Spr_Get_Y(Spr_Number)
  return byte[sprite_y_adr-Spr_Number]

pub Spr_Set_X(Spr_Number, value, increase)
  if increase
    byte[sprite_x_adr-Spr_Number] += value
  else
    byte[sprite_x_adr-Spr_Number] := value
    
pub Spr_Set_Y(Spr_Number, value, increase)
  if increase
    byte[sprite_y_adr-Spr_Number] += value
  else
    byte[sprite_y_adr-Spr_Number] := value

pub Spr_Get_Image(Spr_Number)
  return byte[sprite_number-Spr_Number]  

pub Spr_Set_Image(Spr_Number, value)
  byte[sprite_number-Spr_Number] := value  

dat
long 0
VgaDriver file "waitvid.256x192.driver.2048.dat" '"vgas.cog"
          
DAT
tvparams                long    0               'status
                        long    1               'enable
                        long    %011_0000       'pins ' PROTO/DEMO BOARD = %001_0101 ' HYDRA = %011_0000
                        long    0               'mode - default to NTSC
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    50_000_000'_xinfreq<<4  'broadcast
                        long    0               'auralcog
                        long    SCANLINE_BUFFER
                        long    border_color 'pointer to border colour
                        long    request_scanline
disp_ptr                long    0
VSync                   long    0
                        

nextline                long    0

  'vga_params falls after rend_params
  vga_params              long    0               'sync
  vga_pixels              long    SCANLINE_BUFFER '@pixelsdata+$10 'pointer to line buffers
  vga_nextlineptr         long    request_scanline  'pointer to nextline to be draw
  vga_enable              long    1               'enable display
  vga_cogs                long    1        'number of display lines before looping to top of buffer


                        